/*
	SRAM demo for PCI-M512 SRAM battery backup card.

   OS       : MS-DOS with DOS/4GW extender. (32-bit)
   CPU      : 80386 or higher
   Compiler : Open Watcom C/C++ v1.3

	This program provides some functions for testes, they are...
	. Memory dump function.
	. Erase all SRAM memory.
	. Fill patterns.
	. SRAM memory test.
*/

#include "dos.h"
#include "PCIM512.h"

#define SRAM_M512_SIZE  0x80000  // PCI-M512 512KB (512 * 1024)
#define MAX_COLUMN 16  // Total columns display on screen
#define MAX_ROW    23  // Total rows display on screen
#define BOARD_ACT  1   // Board number to active

UCHAR old_char[1024];  // Buffer used to store old data, and used restore it later

int TestBank(DWORD dwBoard, DWORD i, UCHAR c);
int TestMem(DWORD dwBoard);
int MemDump(DWORD dwBoard);

int main()
{
	DWORD dwRtn, dwBoards; // Return code, total-boards found by driver
	DWORD iOffset, j, iData;
	UCHAR c;

	// *************************************
	// Initialize the driver and
	// returns the number of total-boards found by driver.
	dwRtn = PCIM512_DriverInit(&dwBoards);
	if ( dwRtn )
	{
		printf("Driver initialize error! Error Code=%d\n", (int)dwRtn);
		return 1;
	}
	if (dwBoards < 1)
	{
		printf("Card not found!\n");
		return 1;
	}

	// *************** loop for our testes *******************
	while (1)
	{
		printf("\n\n");
		printf("Please select a function...\n");
		printf("0: Dump SRAM memory.\n");
		printf("1: Erase all SRAM memory.(Lost all data)\n");
		printf("2: Fill SRAM memory with sequence number.(Lost all data)\n");
		printf("3: Fill SRAM memory with address number.(Lost all data)\n");
		printf("4: Test SRAM memory with read/write/compare.\n");
		printf("Q: Exit the program.\n");
		c = getch();

		switch (c) {
		case '0': /******* Dump all SRAM memory *******/
			MemDump(BOARD_ACT);
			break;
		case '1':  /**** Erase all SRAM memory ****/
			for (iOffset=0; iOffset<SRAM_M512_SIZE; iOffset++)
			{
				PCIM512_WriteSramByte(BOARD_ACT, iOffset, 0); // Board-1, offset, data
			}
			printf("Function 1 OK!\n");
			break;
		case '2':  /* Fill SRAM memory with sequence number */
			for (iOffset=0, j=0; iOffset<SRAM_M512_SIZE; iOffset++, j++)
			{
				if (j > 0xFF) j=0;  // j is the sequence number between 0~0xFF
				PCIM512_WriteSramByte(BOARD_ACT, iOffset, j);
			}
			printf("Function 2 OK!\n");
			break;
		case '3':  /* Fill SRAM memory with address number  */
			for (iOffset=0; iOffset<SRAM_M512_SIZE; iOffset+=0x10)
			{
				iData = iOffset;
				for (j=0x10;j>0; j--)
				{
					PCIM512_WriteSramByte(BOARD_ACT, iOffset+j-1, iData);
					iData >>= 8;
					if ( !iData )
						iData = iOffset;
				}
			}
			printf("Function 3 OK!\n");
			break;
		case '4': /******* Test all SRAM memory ******/
			TestMem(BOARD_ACT);
			break;
		case 'q': case 'Q':  /*** Exit the program ***/
			c = 'q';
			break;
		default:
			printf("\n\nNothing selected!\n");
			break;
		}
		if ( c == 'q' )
			break;
	}

	// ************************************
	// Close driver before exit the program
	PCIM512_DriverClose();
	return 0;
}

// **********************************************************************
// Test a single bank of the SRAM, each bank is 1024 bytes
// dwBoard : [IN] Board number to access, base 1
// iBank   : [IN] Bank number to test (1024 bytes), base 0
// c       : [IN] Special char (8-bit) to write/read/compare on the bank.
int TestBank(DWORD dwBoard, DWORD iBank, UCHAR c)
{
	DWORD  jOffset;
	UCHAR  r;

	for (jOffset=0; jOffset<1024; jOffset++)
	{
		PCIM512_WriteSramByte(dwBoard, iBank*1024 +jOffset, c);
	}
	for (jOffset=0; jOffset<1024; jOffset++)
	{
		PCIM512_ReadSramByte(dwBoard, iBank*1024 +jOffset, &r);
		if (r != c)
		{
			printf("SRAM Err at 0x%lX, W:%02X, R:%02X\n", (DWORD)(iBank*1024 +jOffset), (int)c, (int)r);
			//getch();
			return 1;
		}
	}
	return 0;
}

// **********************************************
// Test all SRAM memory
// dwBoard : [IN] Board number to access, base 1
int TestMem(DWORD dwBoard)
{
	DWORD iBank, jOffset;
	for (iBank=0; iBank<512; iBank++)
	{
		for(jOffset=0; jOffset<1024; jOffset++) // backup data
		{
			PCIM512_ReadSramByte(dwBoard, iBank*1024 + jOffset, &(old_char[jOffset]));
		}
		printf("Testing bank %d\n", iBank);
		TestBank(dwBoard, iBank, 0xFF);
		TestBank(dwBoard, iBank, 0xCC);
		TestBank(dwBoard, iBank, 0xAA);
		TestBank(dwBoard, iBank, 0x55);
		TestBank(dwBoard, iBank, 0x33);
		TestBank(dwBoard, iBank, 0x00);

		for(jOffset=0; jOffset<1024; jOffset++) // restore data
		{
			PCIM512_WriteSramByte(dwBoard, iBank*1024 + jOffset, old_char[jOffset] );
		}
	}
	return 0;
}

// ******************************************************
// Dump all SRAM memory
// dwBoard : [IN] Board number to access, base 1
int MemDump(DWORD dwBoard)
{
	DWORD iOffset, dwRtn; // Memory offset, return code
	UCHAR c;              // char (8-bit) data
	int   iPos;           // Screen Position

	c=1;
	printf("\n0x0000 0000: ");  // starting offset is zero
	for (iOffset=0, iPos=0; iOffset<SRAM_M512_SIZE; iOffset++, iPos++)
	{
		// Check if we are going to write a '-' between 8 bytes
		if ( iPos % MAX_COLUMN == 0 )
		{
		   printf("\n0x%04X %04X: ", (WORD)((iOffset)>>16), (WORD)(0xFFFF & (iOffset)) );
		} else if ( iPos % (MAX_COLUMN/2) == 0 ) {
			// Check if we are going to change the row
			printf("- ");
		}

		dwRtn = PCIM512_ReadSramByte(dwBoard, iOffset, &c);
		if ( dwRtn )
		{
			printf("ReadSramByte() Error! Error Code=%d\n", (WORD)dwRtn);
			return 1;
		}
		printf("%02X ", c);

		// Check if we are going to change page
		if ( iPos == MAX_COLUMN * MAX_ROW -1 )
		{
			printf("\nPress 'Q' to quit, 'S' to next segment, any key to continue!\n");
			c = getch();
			iPos = -1;
			if ( c == 'q' || c == 'Q')
				break;
			if ( c == 's' || c == 'S')
			{
				iOffset += MAX_COLUMN * MAX_ROW * 10;
				if (iOffset > SRAM_M512_SIZE)
					iOffset = 0;
				continue;
			}
			//printf("\n");
		}
	}
	return 0;
}

